package com.xy.config;

import com.dtp.core.support.DynamicTp;
import com.xy.utils.ThreadPoolUtils;
import org.springframework.context.annotation.Bean;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;

@Component
public class SysThreadPoolConfig {

    public static final String LOG_POLL = "logPoll";

    public static final String MQTT_CONSUME_ERROR_DATA = "mqttConsumeErrorData";

    /**
     * 日志线程池
     */
    @DynamicTp
    @Bean(LOG_POLL)
    public ThreadPoolTaskExecutor logPoll() {
        int coreSize = 10;
        return ThreadPoolUtils.newPoll()
                .name(LOG_POLL)
                .coreSize(coreSize)
                .maxSize(coreSize * 2)
                .keepAlive(60)
                .queueSize(coreSize * 10)
                .builder();
    }

    /**
     * mqtt消费失败数据线程池
     *
     * @return
     */
    @DynamicTp
    @Bean(MQTT_CONSUME_ERROR_DATA)
    public ThreadPoolTaskExecutor mqttConsumeErrorData() {
        int coreSize = 2;
        return ThreadPoolUtils.newPoll()
                .name(MQTT_CONSUME_ERROR_DATA)
                .coreSize(coreSize)
                .maxSize(coreSize * 2)
                .keepAlive(60)
                .queueSize(coreSize * 10)
                .builder();
    }
}
