package com.xy.service;

import cn.easyes.core.biz.EsPageInfo;
import cn.easyes.core.conditions.LambdaEsQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.xy.collections.list.JArrayList;
import com.xy.dto.LogOperateDto;
import com.xy.entity.LogOperate;
import com.xy.entity.UserInfo;
import com.xy.mapper.LogOperateMapper;
import com.xy.utils.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.List;

import static com.xy.utils.Beans.copy;
import static com.xy.utils.EsBeans.toPageBean;

@Service
@AllArgsConstructor
@Api(tags = "操作日志")
public class LogOperateServiceImpl implements LogOperateService {

    private LogOperateMapper logOperateMapper;

    private UserInfoServiceImpl userInfoService;

    @PostMapping("page")
    @ApiOperation("分页查询")
    public R<PageBean<LogOperateDto.Vo>> page(@RequestBody LogOperateDto.Page page) {
        PageBean pageBean = page.getPage();
        LambdaEsQueryWrapper<LogOperate> lambdaEsQueryWrapper = new EsQueryWrapper().eqWrapper(page, LogOperate.class)
                .ge(LogOperate::getCreateTime, page.getBeginCreateTime())
                .le(LogOperate::getCreateTime, page.getEndCreateTime())
                .like(LogOperate::getLogContent)
                .build()
                .orderByDesc(LogOperate::getCreateTime);
        EsPageInfo<LogOperate> esPageInfo = logOperateMapper.pageQuery(lambdaEsQueryWrapper, (int) pageBean.getCurrent(), (int) pageBean.getSize());
        PageBean<LogOperateDto.Vo> voPageBean = toPageBean(LogOperateDto.Vo.class, esPageInfo);
        List<LogOperateDto.Vo> records = voPageBean.getRecords();
        if (!Emptys.check(records)) {
            return R.ok(voPageBean);
        }
        //外键信息
        copy(records).target(() -> userInfoService.list(new LambdaQueryWrapper<UserInfo>().in(UserInfo::getAuthorizeUserId, new JArrayList<>(records).getProperty(LogOperateDto.Vo::getOptUserId).comparing())),
                LogOperateDto.Vo::getOptUserId, LogOperateDto.Vo::getOptUserName, UserInfo::getAuthorizeUserId, UserInfo::getName).builder();
        return R.ok(voPageBean);
    }

    @PostMapping("mqttBySn")
    @ApiOperation("查询mqtt认证记录")
    public R<List<LogOperateDto.Vo>> mqttBySn(@RequestBody LogOperateDto.MqttBySn mqttBySn) {
        LambdaEsQueryWrapper<LogOperate> lambdaEsQueryWrapper = new LambdaEsQueryWrapper<LogOperate>()
                .like(LogOperate::getLogContent, "mqtt认证查询-结果=false,sn=")
                .distinct(LogOperate::getLogContent)
                .orderByDesc(LogOperate::getCreateTime);
        if (Emptys.check(mqttBySn.getBeginCreateTime())) {
            lambdaEsQueryWrapper.ge(LogOperate::getCreateTime, DataTime.toString(mqttBySn.getBeginCreateTime()));
        }
        if (Emptys.check(mqttBySn.getEndCreateTime())) {
            lambdaEsQueryWrapper.le(LogOperate::getCreateTime, DataTime.toString(mqttBySn.getEndCreateTime()));
        }
        List<LogOperate> logOperates = logOperateMapper.selectList(lambdaEsQueryWrapper);
        return R.ok(copy(LogOperateDto.Vo.class, logOperates));
    }

}
