package com.xy.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.dto.SysRegionDto;
import com.xy.entity.SysRegion;
import com.xy.mapper.SysRegionMapper;
import com.xy.utils.MybatisPlusQuery;
import com.xy.utils.PageBean;
import com.xy.utils.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.ArrayList;
import java.util.List;

import static com.xy.utils.Beans.copy;
import static com.xy.utils.PlusBeans.toIPage;
import static com.xy.utils.PlusBeans.toPageBean;

/**
 * <p>
 * 区域表 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2022-12-28
 */
@Slf4j
@Service
@Api(tags = "区域")
@AllArgsConstructor
public class SysRegionServiceImpl extends ServiceImpl<SysRegionMapper, SysRegion> implements SysRegionService {

    @Override
    @ApiOperation("对象查询")
    public R<SysRegionDto.Vo> object(SysRegionDto.Vo vo) {
        SysRegionDto.SelectList selectList = copy(SysRegionDto.SelectList.class, vo);
        List<SysRegionDto.Vo> list = list(selectList).getData();
        return R.ok(list.size() > 0 ? list.get(0) : null);
    }

    @Override
    @ApiOperation("集合查询")
    public R<List<SysRegionDto.Vo>> list(SysRegionDto.SelectList selectList) {
        if (selectList.getId() != null) {
            selectList.putIds(selectList.getId());
        }
        LambdaQueryWrapper<SysRegion> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(selectList, SysRegion.class)
                .in(SysRegion::getId, selectList.getIds())
                .like(SysRegion::getShortname)
                .like(SysRegion::getName)
                .like(SysRegion::getMergename)
                .like(SysRegion::getPinyin)
                .build()
                .eq(SysRegion::getStatus, 0);
        List<SysRegion> list = list(lambdaQueryWrapper);
        return R.ok(copy(SysRegionDto.Vo.class, list));
    }

    @Override
    @ApiOperation("下级列表")
    public R<List<SysRegionDto.NextListVo>> nextList(SysRegionDto.NextListVoDto dto) {
        LambdaQueryWrapper<SysRegion> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(dto, SysRegion.class)
                .build()
                .eq(SysRegion::getStatus, 0);
        List<SysRegion> list = list(lambdaQueryWrapper);
        return R.ok(copy(SysRegionDto.NextListVo.class, list));
    }

    @PostMapping("page")
    @ApiOperation("分页查询")
    public R<PageBean<SysRegionDto.Vo>> page(@RequestBody SysRegionDto.Page page) {
        LambdaQueryWrapper<SysRegion> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(page, SysRegion.class)
                .like(SysRegion::getShortname)
                .like(SysRegion::getName)
                .like(SysRegion::getMergename)
                .like(SysRegion::getPinyin)
                .build();
        IPage<SysRegion> iPage = page(toIPage(page.getPage()), lambdaQueryWrapper);
        return R.ok(toPageBean(SysRegionDto.Vo.class, iPage));
    }

    @PostMapping("save")
    @ApiOperation("添加")
    public R save(@RequestBody SysRegionDto.Vo vo) {
        SysRegion sysRegion = copy(SysRegion.class, vo);
        save(sysRegion);
        return R.ok();
    }

    @PostMapping("update")
    @ApiOperation("修改")
    public R update(@RequestBody SysRegionDto.Update update) {
        SysRegion sysRegion = copy(SysRegion.class, update);
        updateById(sysRegion);
        return R.ok();
    }


    @PostMapping("createLocalJson")
    @ApiOperation("生成json")
    public R createLocalJson() {
        LambdaQueryWrapper<SysRegion> lqw = new LambdaQueryWrapper<>();
        lqw.select(SysRegion::getId, SysRegion::getPid, SysRegion::getName);
        List<SysRegion> list = list(lqw);
        List<SysRegionDto.LocalJson> localJsonList = new ArrayList<>();
        list.forEach(item -> {
            localJsonList.add(new SysRegionDto.LocalJson().setName(item.getName()).setValue(item.getId().toString()).setParent(item.getPid().toString()));
        });
        return R.ok(localJsonList);
    }
}
