package com.xy.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.dto.UiGuideDto;
import com.xy.entity.UiGuide;
import com.xy.mapper.UiGuideMapper;
import com.xy.utils.Emptys;
import com.xy.utils.MybatisPlusQuery;
import com.xy.utils.PageBean;
import com.xy.utils.R;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.List;

import static com.xy.utils.PlusBeans.*;

/**
 * <p>
 * 界面-引导提示配置表 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2023-01-05
 */
@Service
@Api(tags = "界面-引导提示配置")
public class UiGuideServiceImpl extends ServiceImpl<UiGuideMapper, UiGuide> implements UiGuideService {

    @PostMapping("save")
    @ApiOperation("添加")
    public R save(@RequestBody @Validated UiGuideDto.Save save) {
        long count = count(new LambdaQueryWrapper<UiGuide>().eq(UiGuide::getGuideId, save.getGuideId()));
        if (count > 0) {
            return R.fail("编码已存在");
        }
        LocalDateTime now = LocalDateTime.now();
        UiGuide uiGuide = copy(UiGuide.class, save)
                .setCreateTime(now)
                .setUpdateTime(now);
        save(uiGuide);
        return R.ok();
    }

    @PostMapping("update")
    @ApiOperation("修改")
    public R update(@RequestBody @Validated UiGuideDto.Update update) {
        UiGuide uiGuideInfo = getById(update.getId());
        if (!uiGuideInfo.getGuideId().equals(update.getGuideId())) {
            long count = count(new LambdaQueryWrapper<UiGuide>().eq(UiGuide::getGuideId, update.getGuideId()));
            if (count > 0) {
                return R.fail("编码已存在");
            }
        }
        UiGuide uiGuide = copy(UiGuide.class, update)
                .setCreateTime(LocalDateTime.now());
        updateById(uiGuide);
        return R.ok();
    }

    @PostMapping("del")
    @ApiOperation("删除")
    public R del(@RequestBody @Validated UiGuideDto.Del del) {
        removeBatchByIds(del.getId());
        return R.ok();
    }

    @PostMapping("page")
    @ApiOperation("分页查询")
    public R<PageBean<UiGuideDto.Vo>> page(@RequestBody UiGuideDto.Page page) {
        PageBean pageBean = page.getPage();
        LambdaQueryWrapper<UiGuide> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(page, UiGuide.class)
                .ge(UiGuide::getCreateTime, page.getBeginCreateTime())
                .le(UiGuide::getCreateTime, page.getEndCreateTime())
                .build()
                .orderByDesc(!Emptys.check(pageBean.getOrders()), UiGuide::getCreateTime);
        IPage<UiGuide> iPage = page(toIPage(pageBean), lambdaQueryWrapper);
        return R.ok(toPageBean(UiGuideDto.Vo.class, iPage));
    }

    @PostMapping("list")
    @ApiOperation("集合查询")
    public R<List<UiGuideDto.Vo>> list(@RequestBody UiGuideDto.Vo vo) {
        LambdaQueryWrapper<UiGuide> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(vo, UiGuide.class)
                .eq(UiGuide::getGuideState, 1)
                .build();
        List<UiGuide> list = list(lambdaQueryWrapper);
        return R.ok(copy(UiGuideDto.Vo.class, list));
    }

    @PostMapping("obj")
    @ApiOperation("对象查询")
    public R<UiGuideDto.Vo> obj(@RequestBody UiGuideDto.Vo vo) {
        List<UiGuideDto.Vo> data = list(vo).getData();
        if (Emptys.check(data)) {
            return R.ok(data.get(0));
        }
        return R.ok();
    }
}
