package com.xy.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.yitter.idgen.YitIdHelper;
import com.xy.annotation.Lock;
import com.xy.config.WebMqttConfig;
import com.xy.consts.CommConsts;
import com.xy.consts.SysConstant;
import com.xy.dto.SysWorkUser.AddDto;
import com.xy.dto.SysWorkUser.DelDto;
import com.xy.dto.SysWorkUser.ListByRoleDto;
import com.xy.dto.SysWorkUser.UpdateDto;
import com.xy.dto.UserInfoDto;
import com.xy.entity.UserInfo;
import com.xy.mapper.UserInfoMapper;
import com.xy.utils.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import static com.xy.utils.Beans.copy;
import static com.xy.utils.PlusBeans.toIPage;
import static com.xy.utils.PlusBeans.toPageBean;


/**
 * <p>
 * 用户表 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2023-01-12
 */
@Slf4j
@Service
@AllArgsConstructor
@Api(tags = "用户表")
public class UserInfoServiceImpl extends ServiceImpl<UserInfoMapper, UserInfo> implements UserInfoService {

    private SysWorkUserService sysWorkUserService;

    private WebMqttConfig webMqttConfig;


    @ApiOperation("获取所有售后权限用户ID")
    @Override
    public R<UserInfoDto.SaleBackUsersVO> getSaleBackUsers() {
        UserInfoDto.SaleBackUsersVO vo = new UserInfoDto.SaleBackUsersVO();
        ListByRoleDto listByRoleDto = new ListByRoleDto();
        listByRoleDto.setRoleCode(CommConsts.MERC_SH_ADMIN);
        listByRoleDto.setSysId(SysConstant.BIZ_SYS_ID);
        List<Long> userIds = R.feignCheckData(sysWorkUserService.getUserIdListByRole(listByRoleDto));
        //所有售后管理员及管理员创建的用户
        List<UserInfo> userInfoList = this.list(Wrappers.<UserInfo>lambdaQuery()
                .and(deviceAlgorithmChargingLambdaQueryWrapper -> deviceAlgorithmChargingLambdaQueryWrapper
                        .in(UserInfo::getCreateUser, userIds)
                        .or()
                        .in(UserInfo::getAuthorizeUserId, userIds)
                )
        );
        if (CollUtil.isNotEmpty(userInfoList)) {
            List<Long> ids = userInfoList.stream().map(UserInfo::getAuthorizeUserId).distinct().collect(Collectors.toList());
            List<String> wxMpOpenIds = userInfoList.stream().map(UserInfo::getMpOpenid).distinct().collect(Collectors.toList());
            vo.setAuthUserIds(ids);
            vo.setMpOpenIds(wxMpOpenIds);
        }
        return R.ok(vo);
    }

    @Override
    @ApiOperation("集合查询")
    public R<List<UserInfoDto.Vo>> list(UserInfoDto.SelectListDto selectList) {
        List<Long> userIds = selectList.getUserIds();
        UserInfoDto.Vo userVo = selectList.getUserVo();
        LambdaQueryWrapper<UserInfo> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        if (userVo != null) {
            lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(selectList.getUserVo(), UserInfo.class).build();
        }
        lambdaQueryWrapper.in(CollUtil.isNotEmpty(userIds), UserInfo::getUserId, userIds);
        List<UserInfo> list = list(lambdaQueryWrapper);
        return R.ok(copy(UserInfoDto.Vo.class, list));
    }


    @Override
    @ApiOperation("对象查询")
    public R<UserInfoDto.Vo> obj(UserInfoDto.Vo vo) {
        LambdaQueryWrapper<UserInfo> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(vo, UserInfo.class).build();
        List<UserInfo> list = list(lambdaQueryWrapper);
        if (Emptys.check(list)) {
            return R.ok(copy(UserInfoDto.Vo.class, list.get(0)));
        }
        return R.ok();
    }

    @PostMapping("page")
    @ApiOperation("分页查询")
    public R<PageBean<UserInfoDto.Vo>> page(@RequestBody UserInfoDto.Page page) {
        PageBean pageBean = page.getPage();
        LambdaQueryWrapper<UserInfo> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(page, UserInfo.class)
                .like(UserInfo::getTel, page.getTel())
                .like(UserInfo::getMail, page.getMail())
                .like(UserInfo::getUnionid, page.getUnionid())
                .ge(UserInfo::getCreateTime, page.getBeginCreateTime())
                .le(UserInfo::getCreateTime, page.getEndCreateTime())
                .build()
                .eq(!AuthorizeUtils.authByData(page.getSysId()), UserInfo::getCreateUser, AuthorizeUtils.getLoginId(Long.class))
                .orderByDesc(!Emptys.check(pageBean.getOrders()), UserInfo::getCreateTime);
        IPage<UserInfo> iPage = page(toIPage(pageBean), lambdaQueryWrapper);
        return R.ok(toPageBean(UserInfoDto.Vo.class, iPage));
    }

    @Override
    @ApiOperation("添加")
    @Lock(value = "save.tel", prefix = "user_save_")
    public R<UserInfoDto.Vo> save(UserInfoDto.Save save) {
        //添加权限用户
        AddDto addDto = copy(AddDto.class, save)
                .setEmail(save.getMail())
                .setPhone(save.getTel())
                .setRoleIds(save.getRoleIds())
                .setAccount(save.getAccount());
        R<Long> register = sysWorkUserService.register(addDto);
        if (register.getCode() != R.Enum.SUCCESS.getCode()) {
            return R.fail(register.getMsg());
        }
        Long authorizeUserId = register.getData();
        //添加用户
        Long loginId = AuthorizeUtils.getLoginId(Long.class);
        UserInfo saveInfo = copy(UserInfo.class, save)
                .createUserTime(loginId)
                .setUserId(YitIdHelper.nextId())
                .setAuthorizeUserId(authorizeUserId)
                .createUserTime(loginId);
        save(saveInfo);
        return R.ok(copy(UserInfoDto.Vo.class, saveInfo));
    }

    @Override
    @ApiOperation("修改")
    @Lock(value = "update.userId", prefix = "user_update_")
    public R update(UserInfoDto.Update update) {
        log.info("权限用户信息修改请求：{}", JSONUtil.toJsonPrettyStr(update));
        UserInfo byId = getById(update.getUserId());
        String account = update.getAccount();

        if (StrUtil.isNotEmpty(update.getTel())) {
            if (!update.getTel().equals(byId.getTel())) {
                long count = count(new LambdaQueryWrapper<UserInfo>().eq(UserInfo::getTel, update.getTel()));
                if (count > 0) {
                    return R.fail("手机已存在");
                }
            }
        }
        if (StrUtil.isNotEmpty(update.getMail())) {
            if (!byId.getMail().equals(update.getMail())) {
                long count = count(new LambdaQueryWrapper<UserInfo>().eq(UserInfo::getMail, update.getMail()));
                if (count > 0) {
                    return R.fail("邮箱已存在");
                }
            }
        }
        UserInfo updateInfo = copy(UserInfo.class, update)
                .updateUserTime(AuthorizeUtils.getLoginId(Long.class));
        UserInfo u = getById(update.getUserId());
        //修改权限用户
        if (Emptys.check(update.getTel()) || Emptys.check(update.getMail()) || Emptys.check(update.getStatus()) || Emptys.check(update.getPassword())) {
            UpdateDto updateDto = new UpdateDto()
                    .setId(u.getAuthorizeUserId())
                    .setAccount(account)
                    .setPhone(update.getTel())
                    .setEmail(update.getMail())
                    .setPassword(update.getPassword())
                    .setStatus(update.getStatus())
                    .setRoleIds(update.getRoleIds());
            log.info("权限用户信息修改：{}", JSONUtil.toJsonPrettyStr(updateDto));
            sysWorkUserService.update(updateDto);
        }
        //修改用户
        updateById(updateInfo);
        return R.ok();
    }

    @ApiOperation("删除用户")
    @Override
    public R del(@RequestBody @Validated UserInfoDto.DelDto dto) {
        //sys_work_user 权限用户表
        R r = sysWorkUserService.del(copy(DelDto.class, dto));
        if (R.Enum.SUCCESS.getCode() != r.getCode()) {
            return R.fail(r.getMsg());
        }
        //user_info 用户表
        return R.ok(removeBatchByIds(dto.getId()));
    }

    @PostMapping("webUserMqtt")
    @ApiOperation("获取用户mqtt信息")
    public R<UserInfoDto.WebUserMqttVo> webUserMqtt(@RequestBody @Validated UserInfoDto.WebUserMqtt webUserMqtt) {
        Long loginId = AuthorizeUtils.getLoginId(Long.class);
        Integer type = webUserMqtt.getType();
        UserInfoDto.WebUserMqttVo webUserMqttVo = new UserInfoDto.WebUserMqttVo()
                .setUsername(webMqttConfig.getUsername())
                .setPassword(webMqttConfig.getPassword());
        WebMqttConfig.Protocol protocol = webMqttConfig.getProtocol();
        WebMqttConfig.Topic topic = webMqttConfig.getTopic();
        if (type == 1) {
            webUserMqttVo.setUrl(protocol.getPcUrl() + webMqttConfig.getIp())
                    .setPort(webMqttConfig.getWsPort())
                    .setClientTopic(topic.getPrefix() + topic.getPcManagePrefix() + loginId);
        } else if (type == 2) {
            webUserMqttVo.setUrl(protocol.getWxsUrl() + webMqttConfig.getIp())
                    .setPort(webMqttConfig.getWssPort())
                    .setClientTopic(topic.getPrefix() + topic.getWxCPrefix() + loginId);
        } else if (type == 3) {
            webUserMqttVo.setUrl(protocol.getAlisUrl() + webMqttConfig.getIp())
                    .setPort(webMqttConfig.getWssPort())
                    .setClientTopic(topic.getPrefix() + topic.getAlisCPrefix() + loginId);
        }
        return R.ok(webUserMqttVo);
    }

    @ApiOperation("修改微信信息")
    @Override
    public R updateByWechat(UserInfoDto.UpdateByWechat dto) {
        updateById(copy(UserInfo.class, dto));
        return R.ok();
    }

    @ApiOperation("修改支付宝信息")
    @Override
    public R updateByAli(UserInfoDto.UpdateByAli dto) {
        updateById(copy(UserInfo.class, dto));
        return R.ok();
    }


}
