package com.xy.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.xy.utils.PageBean;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotEmpty;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 *
 * </p>
 *
 * @author 谭斌
 * @since 2023-02-24
 */
public class MsgUserMessageDto {

    @Data
    @Accessors(chain = true)
    public static class ReadOver {
        @NotEmpty(message = "消息ID不可为空")
        @ApiModelProperty(value = "消息ID", required = true)
        private List<Long> ids;
    }


    @Data
    @Accessors(chain = true)
    public static class SelectList extends Vo {


    }

    @Data
    @Accessors(chain = true)
    public static class Page extends Vo {

        @ApiModelProperty(value = "类型(1故障通知、2交易通知、3其他通知)", required = false)
        private String type;

        @ApiModelProperty(value = "分页对象", required = true)
        private PageBean page;

        @JsonFormat(pattern = "yyyy-MM-dd")
        @ApiModelProperty(value = "创建时间-起")
        private LocalDate beginCreateTime;

        @JsonFormat(pattern = "yyyy-MM-dd")
        @ApiModelProperty(value = "创建时间-始")
        private LocalDate endCreateTime;

        public LocalDateTime getBeginCreateTime() {
            return beginCreateTime == null ? null : beginCreateTime.atTime(0, 0, 0);
        }

        public LocalDateTime getEndCreateTime() {
            return endCreateTime == null ? null : endCreateTime.atTime(23, 59, 59);
        }

    }

    @Data
    @Accessors(chain = true)
    public static class Save {
        @ApiModelProperty(value = "用户id")
        private Long userId;

        @ApiModelProperty(value = "消息id")
        private Integer msgId;

        @ApiModelProperty(value = "状态")
        private String readState;


        @ApiModelProperty(value = "消息类型")
        private Integer msgType;

        /**
         * 1紧急 2重要 3一般 (默认)
         */
        @ApiModelProperty("优先级")
        private Integer priority;


        /**
         * 是否弹出显示
         */
        @ApiModelProperty(value = "是否弹出显示")
        private Boolean popUp;
    }

    @Data
    @Accessors(chain = true)
    public static class Update extends Vo {

    }

    @Data
    @Accessors(chain = true)
    public static class PageCount {
        @ApiModelProperty(value = "全部未读数")
        private Integer allNum;
        @ApiModelProperty(value = "交易未读数")
        private Integer tradeNum;
        @ApiModelProperty(value = "故障未读数")
        private Integer riskNum;
        @ApiModelProperty(value = "其他未读数")
        private Integer otherNum;
        @ApiModelProperty(value = "费用未读数")
        private Integer costNum;
    }

    @Data
    @Accessors(chain = true)
    public static class PageVO {


        @ApiModelProperty(value = "消息类型（字段msg_types）")
        private String msgType;

        @ApiModelProperty(value = "消息类型")
        private String title;

        @ApiModelProperty(value = "内容")
        private String content;

        @ApiModelProperty("优先级 1紧急 2重要 3一般")
        private Integer priority;

        @ApiModelProperty(value = "状态（2已读、1未读）")
        private String readState;


        @ApiModelProperty(value = "id")
        private Long id;

        @ApiModelProperty(value = "用户id")
        private Long userId;


        @ApiModelProperty(value = "消息id")
        private Integer msgId;


        @ApiModelProperty(value = "发送时间")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime createTime;


        /**
         * 是否弹出显示
         */
        @ApiModelProperty(value = "是否弹出显示")
        private Boolean popUp;

    }

    @Data
    @Accessors(chain = true)
    public static class Vo {

        @ApiModelProperty(value = "id")
        private Long id;

        @ApiModelProperty(value = "用户id")
        private Long userId;

        @ApiModelProperty(value = "消息id")
        private Integer msgId;

        @ApiModelProperty(value = "状态")
        private String readState;

        @ApiModelProperty(value = "创建时间")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime createTime;

        @ApiModelProperty(value = "消息类型")
        private String msgType;

        /**
         * 1紧急 2重要 3一般 (默认)
         */
        @ApiModelProperty("优先级")
        private Integer priority;

        @ApiModelProperty(value = "消息标题")
        private String title;

        @ApiModelProperty(value = "内容")
        private String content;


        /**
         * 是否弹出显示
         */
        @ApiModelProperty(value = "是否弹出显示")
        private Boolean popUp;


    }

}
