package com.xy.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.xy.utils.PageBean;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import java.time.LocalDateTime;
import java.util.List;

/**
 * <p>
 * 系统编码配置表
 * </p>
 *
 * @author lijin
 * @since 2023-04-20
 */
public class SysCodeConfigureDto {

    @Data
    @Accessors(chain = true)
    public static class SelectList extends Vo {

        @ApiModelProperty(value = "编码")
        private List<String> codes;
    }

    @Data
    @Accessors(chain = true)
    public static class Page extends Vo {

        @ApiModelProperty(value = "分页对象", required = true)
        private PageBean page;

    }

    @Data
    @Accessors(chain = true)
    public static class Save extends Vo {

        @NotBlank(message = "code不能为空")
        @ApiModelProperty(value = "编码")
        private String code;
    }

    @Data
    @Accessors(chain = true)
    public static class Update extends Vo {

        @NotBlank(message = "code不能为空")
        @ApiModelProperty(value = "编码")
        private String code;
    }

    @Data
    @Accessors(chain = true)
    public static class Del {

        @NotEmpty(message = "id不能为空")
        @ApiModelProperty("唯一标识")
        private List<String> code;
    }

    @Data
    @Accessors(chain = true)
    public static class TreeDto {

        @ApiModelProperty(value = "编码")
        private List<String> codes;

    }

    @Data
    @Accessors(chain = true)
    public static class TreeVo extends Vo {

        @ApiModelProperty(value = "子级编码")
        private List<TreeVo> sonVos;
    }

    @Data
    @Accessors(chain = true)
    public static class Vo {

        @ApiModelProperty(value = "编码")
        private String code;

        @ApiModelProperty(value = "上级编码")
        private String paterCode;

        @ApiModelProperty(value = "说明")
        private String msg;

        @ApiModelProperty(value = "标题")
        private String title;

        @ApiModelProperty(value = "建议")
        private String propose;

        @ApiModelProperty(value = "扩展字段")
        private String expand;

        @ApiModelProperty(value = "创建人")
        private Long createUser;

        @ApiModelProperty(value = "创建时间")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime createTime;

        @ApiModelProperty(value = "更新人")
        private Long updateUser;

        @ApiModelProperty(value = "更新时间")
        @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
        private LocalDateTime updateTime;

    }

}
