package com.xy.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.xy.utils.PageBean;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;

@Data
@Accessors(chain = true)
public class UiGuideDto {

    @Data
    @Accessors(chain = true)
    public static class Save {

        @NotBlank(message = "guideId不能为空")
        @ApiModelProperty(value = "引导编码", required = true)
        private String guideId;

        @NotBlank(message = "guideTitle不能为空")
        @ApiModelProperty(value = "引导标题", required = true)
        private String guideTitle;

        @NotNull(message = "guideType不能为空")
        @ApiModelProperty(value = "引导类型", required = true)
        private Integer guideType;

        @NotBlank(message = "guideContent不能为空")
        @ApiModelProperty(value = "引导内容", required = true)
        private String guideContent;

        @NotNull(message = "guideContentType不能为空")
        @ApiModelProperty(value = "引导类容内型", required = true)
        private Integer guideContentType;

        @NotNull(message = "viewType不能为空")
        @ApiModelProperty(value = "显示方式", required = true)
        private Integer viewType;

        @ApiModelProperty("链接")
        private String guideLink;
    }

    @Data
    @Accessors(chain = true)
    public static class Update extends Vo {

        @NotNull(message = "id不能为空")
        @ApiModelProperty(value = "id", required = true)
        private Integer id;
    }

    @Data
    @Accessors(chain = true)
    public static class Del {

        @NotEmpty(message = "id不能为空")
        @ApiModelProperty(value = "id", required = true)
        private List<Integer> id;
    }

    @Data
    @Accessors(chain = true)
    public static class Page extends Vo {

        private PageBean page;

        @JsonFormat(pattern = "yyyy-MM-dd")
        @ApiModelProperty(value = "创建时间-起")
        private LocalDate beginCreateTime;

        @JsonFormat(pattern = "yyyy-MM-dd")
        @ApiModelProperty(value = "创建时间-始")
        private LocalDate endCreateTime;

        public LocalDateTime getBeginCreateTime() {
            return beginCreateTime == null ? null : beginCreateTime.atTime(0, 0, 0);
        }

        public LocalDateTime getEndCreateTime() {
            return endCreateTime == null ? null : endCreateTime.atTime(23, 59, 59);
        }
    }

    @Data
    @Accessors(chain = true)
    public static class Vo {

        @ApiModelProperty("id")
        private Integer id;

        @ApiModelProperty("引导编码")
        private String guideId;

        @ApiModelProperty("引导标题")
        private String guideTitle;

        @ApiModelProperty("引导类型")
        private Integer guideType;

        @ApiModelProperty("启用状态")
        private Integer guideState;

        @ApiModelProperty("引导内容")
        private String guideContent;

        @ApiModelProperty("引导类容内型")
        private Integer guideContentType;

        @ApiModelProperty("显示方式")
        private Integer viewType;

        @ApiModelProperty("链接")
        private String guideLink;

        @ApiModelProperty("更新时间")
        private LocalDateTime updateTime;

        @ApiModelProperty("创建时间")
        private LocalDateTime createTime;
    }

}
