package com.xy.utils;

import cn.hutool.core.collection.CollStreamUtil;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 算法平台工具类
 * </p>
 *
 * @author hechunping
 * @since 2023/3/13
 */
public class AlgorithmUtils {
    private static String REDIS_KEY = "sys:algorithm:";
    /**
     * redis列表KEY
     */
    public static String REDIS_KEY_LIST = REDIS_KEY+"list";
    /**
     * redis列表OBJ
     */
    public static String REDIS_KEY_OBJ = REDIS_KEY+"obj:";


    /**
     * 列表 (所有数据)
     *
     * @return {@link List}<{@link Algorithm}>
     */
    public static List<Algorithm> listAll(){
        RedisService redisService = SpringBeanUtils.getBean(RedisService.class);
        return (List<Algorithm>) redisService.get(REDIS_KEY_LIST);
    }

    /**
     * 列表 (启用中的ID和name)
     *
     * @return {@link List}<{@link Algorithm}>
     */
    public static List<Tuple.Tuple2<Long,String>> listIdName(){
        RedisService redisService = SpringBeanUtils.getBean(RedisService.class);
        List<Algorithm> list = (List<Algorithm>)redisService.get(REDIS_KEY_LIST);
        List<Tuple.Tuple2<Long,String>> listTuple = new ArrayList<>();
        list.forEach(item->{
            if(item.status){
                Tuple.Tuple2<Long,String> idName = new Tuple.Tuple2<>(item.getId(), item.name);
                listTuple.add(idName);
            }
        });
        return listTuple;
    }

    /**
     * 返回ID MAP
     * 所有数据
     * @return
     */

    public static Map<Long,Algorithm> idMap(){
        RedisService redisService = SpringBeanUtils.getBean(RedisService.class);
        List<Algorithm> list = (List<Algorithm>)redisService.get(REDIS_KEY_LIST);
        return CollStreamUtil.toIdentityMap(list, Algorithm::getId);
    }

    /**
     * 对象
     *
     * @param id 算法ID
     * @return {@link Algorithm}
     */
    public static Algorithm obj(Long id){
        RedisService redisService = SpringBeanUtils.getBean(RedisService.class);
        return (Algorithm) redisService.get(REDIS_KEY_OBJ+id.toString());
    }
    /**
     * 算法
     *
     * @author hechunping
     * @date 2023/03/13
     */
    @Data
    @Accessors(chain = true)
    public static class Algorithm{

        private Long id;

        private String name;

        private String host;

        private Integer mercOnceCost;

        private String remark;

        private Boolean status;
    }

}
