package com.xy.utils;

import com.xy.collections.map.JHashMap;
import com.xy.collections.map.JMap;
import com.xy.entity.SysDictRedis;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 字典工具类
 */
public class SysDictUtils {

    /**
     * 字典redis前缀
     */
    public static final String SYS_DICT_REDIS_PREFIX = "sys_dict";

    public static String getKey(String key) {
        return String.format("%s:%s", SYS_DICT_REDIS_PREFIX, key);
    }

    private static RedisService<SysDictRedis> getRedisService() {
        return SpringBeanUtils.getBean(RedisService.class);
    }

    /**
     * 父code获取所有子级列表
     *
     * @param paterCode 父code
     * @return k=code v=子对象
     */
    public static Map<String, SysDictRedis> get(String paterCode) {
        return getRedisService().getMap(getKey(paterCode));
    }

    /**
     * 根据传入字典类型批量获取字典数据
     *
     * @param paterCodes
     * @return
     */
    public static List<Map<String, SysDictRedis>> get(List<String> paterCodes) {
        List<Map<String, SysDictRedis>> list = new ArrayList<>(paterCodes.size());
        RedisService<SysDictRedis> redisService = getRedisService();
        paterCodes.forEach(code -> list.add(redisService.getMap(getKey(code))));
        return list;
    }

    /**
     * 父code和code获取对象
     *
     * @param paterCode 父code
     * @param code      code
     * @return 子对象
     */
    public static SysDictRedis get(String paterCode, String code) {
        return getRedisService().getMap(getKey(paterCode), code);
    }


    /**
     * 获取指定类型编码值
     *
     * @param paterCode
     * @param code
     * @param tClass
     * @param <T>
     * @return
     */
    public static <T> T getValue(String paterCode, String code, Class<T> tClass) {
        SysDictRedis sysDictRedis = get(paterCode, code);
        String value = sysDictRedis.getValue();
        return tClass.equals(Long.class) ? (T) Long.valueOf(value)
                : tClass.equals(Integer.class) ? (T) Integer.valueOf(value)
                : tClass.equals(String.class) ? (T) value
                : (T) value;
    }

    /**
     * 获取所有字典
     *
     * @return
     */
    public static Map<String, Map<String, SysDictRedis>> all() {
        return getRedisService().likeMap(SYS_DICT_REDIS_PREFIX);
    }

    /**
     * 根据label查字典
     *
     * @param paterCode
     * @param label
     * @return
     */
    @Deprecated
    public static SysDictRedis getByLabel(String paterCode, String label) {
        Map<String, SysDictRedis> stringSysDictRedisMap = get(paterCode);
        SysDictRedis sysDict = new SysDictRedis();
        for (Map.Entry<String, SysDictRedis> entry : stringSysDictRedisMap.entrySet()) {
            SysDictRedis sysDictRedis = entry.getValue();
            if (sysDictRedis.getMsg().equals(label)) {
                sysDict = sysDictRedis;
                break;
            }
        }
        return sysDict;
    }

    /**
     * 根据值获取字典
     *
     * @param paterCode
     * @param value
     * @return
     */
    @Deprecated
    public static SysDictRedis getByValue(String paterCode, String value) {
        Map<String, SysDictRedis> stringSysDictRedisMap = get(paterCode);
        JMap<String, SysDictRedis> jMap = new JHashMap<>(stringSysDictRedisMap).toList().value().toMap(SysDictRedis::getValue).cover();
        return jMap.get(value);
    }

    /**
     * 根据值获取字典
     *
     * @param paterCode
     * @param value
     * @return
     */
    @Deprecated
    public static <T> T getByValue(String paterCode, String value, Class<T> tClass) {
        SysDictRedis sysDictRedis = getByValue(paterCode, value);
        return getValue(sysDictRedis.getPaterCode(), sysDictRedis.getCode(), tClass);
    }
}
